from django.core.files.storage import DefaultStorage
from django.core.files.uploadedfile import UploadedFile
from django.db import models
from django.dispatch import receiver


class Template(models.Model):
    DOCX_TEMPLATE = "docx-template"
    XLSX_TEMPLATE = "xlsx-template"
    ENGINE_CHOICES_LIST = (DOCX_TEMPLATE, XLSX_TEMPLATE)
    ENGINE_CHOICES_TUPLE = (
        (
            DOCX_TEMPLATE,
            "Docx Template engine (https://github.com/elapouya/python-docx-template)",
        ),
        (
            XLSX_TEMPLATE,
            "Xlsx Template engine (https://github.com/zhangyu836/python-xlsx-template)",
        ),
    )

    slug: models.SlugField = models.SlugField(primary_key=True)
    description: models.TextField = models.TextField(default="")
    template: models.FileField = models.FileField(max_length=1024)
    engine: models.CharField = models.CharField(
        max_length=20, choices=ENGINE_CHOICES_TUPLE
    )
    created_at: models.DateTimeField = models.DateTimeField(
        auto_now_add=True, db_index=True, null=True
    )
    created_by_user: models.CharField = models.CharField(
        max_length=150, blank=True, null=True
    )
    created_by_group: models.CharField = models.CharField(
        max_length=255, blank=True, null=True
    )
    modified_at: models.DateTimeField = models.DateTimeField(
        auto_now=True, db_index=True, null=True
    )
    modified_by_user: models.CharField = models.CharField(
        max_length=150, blank=True, null=True
    )
    modified_by_group: models.CharField = models.CharField(
        max_length=255, blank=True, null=True
    )
    meta = models.JSONField(default=dict)


@receiver(models.signals.post_delete, sender=Template)
def auto_delete_file_on_delete(sender, instance, **kwargs):
    """Delete template file from filesystem when `Template` object is deleted."""

    if instance.template:
        DefaultStorage().delete(instance.template.name)


@receiver(models.signals.pre_save, sender=Template)
def auto_delete_file_on_change(sender, instance, **kwargs):
    """
    Delete old template file from filesystem when `Template` is given a new template file.

    If a new template file is uploaded, we need to explicitly delete the old file
    to prevent dangling files in the storage. If no new file is being uploaded (i.e.
    only other, normal attributes are changed), we keep the file.
    """

    try:
        old_file = Template.objects.get(pk=instance.pk).template
    except Template.DoesNotExist:
        return

    if old_file:
        new_file = instance.template
        if isinstance(new_file.file, UploadedFile):
            DefaultStorage().delete(old_file.name)



import os

wsgi_app = "document_merge_service.wsgi:application"
bind = f"{os.environ.get('GUNICORN_HOST', '0.0.0.0')}:{os.environ.get('GUNICORN_PORT', 8000)}"
workers = os.environ.get("GUNICORN_WORKERS", 8)
proc_name = "document-merge-service"
timeout = os.environ.get("GUNICORN_TIMEOUT", 60)
limit_request_line = os.environ.get("GUNICORN_LIMIT_REQUEST_LINE", 8190)



from copy import deepcopy
from io import StringIO

import pytest
from django.core.exceptions import ImproperlyConfigured
from django.core.files import File as DjangoFile
from django.core.management import call_command
from storages.backends.s3 import S3Storage

from document_merge_service.api.data import django_file


@pytest.fixture
def settings_storage(settings):
    settings.STORAGES = deepcopy(settings.STORAGES)
    return settings.STORAGES


def test_encrypt_templates(db, settings, settings_storage, mocker, template_factory):
    template_factory(template=django_file("docx-template.docx"))

    settings.DMS_ENABLE_AT_REST_ENCRYPTION = True
    settings_storage["default"] = {
        "BACKEND": "storages.backends.s3.S3Storage",
        "OPTIONS": {
            **settings.S3_STORAGE_OPTIONS,
            "object_parameters": {
                "SSECustomerKey": "x" * 32,
                "SSECustomerAlgorithm": "AES256",
            },
        },
    }

    mocker.patch("storages.backends.s3.S3Storage.open")
    mocker.patch("storages.backends.s3.S3Storage.save")
    S3Storage.save.return_value = "name-of-the-file"
    S3Storage.open.return_value = DjangoFile(open("README.md", "rb"))

    call_command("dms_encrypt_templates")

    assert S3Storage.open.call_count == 1
    assert S3Storage.save.call_count == 1


def test_encrypt_templates_disabled(db, template_factory):
    template_factory(template=django_file("docx-template.docx"))

    out = StringIO()
    call_command("dms_encrypt_templates", stdout=out)

    assert (
        "Encryption is not enabled. Skipping encryption of templates." in out.getvalue()
    )


def test_encrypt_template_improperyconfigured(db, settings, template_factory):
    template_factory(template=django_file("docx-template.docx"))
    settings.DMS_ENABLE_AT_REST_ENCRYPTION = True

    out = StringIO()
    with pytest.raises(ImproperlyConfigured):
        call_command("dms_encrypt_templates", stdout=out)


def test_encrypt_templates_failed(
    db, settings, settings_storage, mocker, template_factory
):
    template_factory(template=django_file("docx-template.docx"))

    settings.DMS_ENABLE_AT_REST_ENCRYPTION = True
    settings_storage["default"] = {
        "BACKEND": "storages.backends.s3.S3Storage",
        "OPTIONS": {
            **settings.S3_STORAGE_OPTIONS,
            "object_parameters": {
                "SSECustomerKey": "x" * 32,
                "SSECustomerAlgorithm": "AES256",
            },
        },
    }

    mocker.patch("storages.backends.s3.S3Storage.open", side_effect=FileNotFoundError)
    mocker.patch("storages.backends.s3.S3Storage.save")
    S3Storage.save.return_value = "name-of-the-file"
    S3Storage.open.return_value = DjangoFile(open("README.md", "rb"))

    out = StringIO()
    call_command("dms_encrypt_templates", stdout=out)

    assert S3Storage.open.call_count == 1
    assert S3Storage.save.call_count == 0
    assert "failed" in out.getvalue()
